import vertex from './shader_vertex.js'
import fragment from './shader_fragment_2.js'

class App {
    constructor() {
        this._initialize();
    }

    _initialize() {
        this.canvas = document.querySelector("#canvas");
        this.gl = this.canvas.getContext("webgl2");

        this.mouseX = 0;
        this.mouseY = 0;

        this._setupShader();
        this._setupEvents();
        this._setupModel();

        this.resize();
    }

    _setupShader() {
        const gl = this.gl;
        const program = webglUtils.createProgramFromSources(gl, [vertex, fragment]);

        gl.useProgram(program);

        this.program = program;
    }

    _setupEvents() {
        window.onresize = this.resize.bind(this);

        canvas.addEventListener('mousemove', e => {
            const rect = canvas.getBoundingClientRect();
            this.mouseX = e.clientX - rect.left;
            this.mouseY = rect.height - (e.clientY - rect.top) - 1;        
        });
    }

    resize() {
        const gl = this.gl;

        webglUtils.resizeCanvasToDisplaySize(gl.canvas);
        gl.viewport(0, 0, gl.canvas.width, gl.canvas.height);
    }

    _setupModel() {
        const gl = this.gl;
        const program = this.program;

        this.positionAttributeLocation = gl.getAttribLocation(program, "a_position");
        this.resolutionLocation = gl.getUniformLocation(program, "u_resolution");
        this.mouseLocation = gl.getUniformLocation(program, "u_mouse");
        this.timeLocation = gl.getUniformLocation(program, "u_time");

        const positionBuffer = gl.createBuffer();

        gl.bindBuffer(gl.ARRAY_BUFFER, positionBuffer);
        gl.bufferData(gl.ARRAY_BUFFER, new Float32Array([
            -1, -1, 1, -1, -1,  1,
            -1,  1, 1, -1,  1,  1,
        ]), gl.STATIC_DRAW);

        gl.enableVertexAttribArray(this.positionAttributeLocation);

        gl.vertexAttribPointer(this.positionAttributeLocation, 2, gl.FLOAT, false, 0, 0);
    }

    render(time) {
        const gl = this.gl;
        time *= 0.0005;

        gl.uniform2f(this.resolutionLocation, gl.canvas.width, gl.canvas.height);
        gl.uniform2f(this.mouseLocation, this.mouseX, this.mouseY);
        gl.uniform1f(this.timeLocation, time);

        gl.drawArrays(gl.TRIANGLES, 0, 6);

        requestAnimationFrame(this.render.bind(this));
    }
}

window.onload = function () {
    (new App()).render(0);
}